### Name: alaplace
### Title: Asymmetric Laplace Distribution Family Functions
### Aliases: alaplace1 alaplace2 alaplace3
### Keywords: models regression

### ** Examples

# Example 1: quantile regression with smoothing splines
alldat = data.frame(x = sort(runif(n <- 500)))
mymu = function(x) exp(-2 + 6*sin(2*x-0.2) / (x+0.5)^2)
alldat = transform(alldat, y = rpois(n, lambda=mymu(x)))
mytau = c(0.25, 0.75); mydof = 4

fit = vgam(y ~ s(x, df=mydof), alaplace1(tau=mytau, llocation="loge",
           parallelLoc=FALSE), data=alldat, trace=TRUE)
fitp = vgam(y ~ s(x, df=mydof), alaplace1(tau=mytau, llocation="loge",
            parallelLoc=TRUE), data=alldat, trace=TRUE)
 
## Not run: 
##D par(las=1)
##D mylwd = 1.5
##D with(alldat, plot(x, jitter(y, factor=0.5), col="red",
##D                   main="Example 1; green: parallelLoc=TRUE",
##D                   ylab="y", pch="o", cex=0.75))
##D with(alldat, matlines(x, fitted(fit), col="blue", lty="solid", lwd=mylwd))
##D with(alldat, matlines(x, fitted(fitp), col="green", lty="solid", lwd=mylwd))
##D finexgrid = seq(0, 1, len=1001)
##D for(ii in 1:length(mytau))
##D     lines(finexgrid, qpois(p=mytau[ii], lambda=mymu(finexgrid)),
##D           col="blue", lwd=mylwd)
## End(Not run)
fit@extra  # Contains useful information

# Example 2: regression quantile at a new tau value from an existing fit
# Nb. regression splines are used here since it is easier.
fitp2 = vglm(y ~ bs(x, df=mydof),
             family = alaplace1(tau=mytau, llocation="loge",
                                parallelLoc=TRUE),
             data=alldat, trace=TRUE)

newtau = 0.5  # Want to refit the model with this tau value
fitp3 = vglm(y ~ 1 + offset(predict(fitp2)[,1]),
            family = alaplace1(tau=newtau, llocation="loge"),
             data=alldat)
## Not run: 
##D with(alldat, plot(x, jitter(y, factor=0.5), col="red", ylab="y",
##D                   pch="o", cex=0.75,
##D                   main="Example 2; parallelLoc=TRUE"))
##D with(alldat, matlines(x, fitted(fitp2), col="blue", lty="solid", lwd=mylwd))
##D with(alldat, matlines(x, fitted(fitp3), col="black", lty="solid", lwd=mylwd))
## End(Not run)


# Example 3: noncrossing regression quantiles using a trick: obtain
# successive solutions which are added to previous solutions; use a log
# link to ensure an increasing quantiles at any value of x.

mytau = seq(0.2, 0.9, by=0.1)
answer = matrix(0, nrow(alldat), length(mytau)) # Stores the quantiles
alldat = transform(alldat, offsety=y*0)
usetau = mytau
for(ii in 1:length(mytau)) {
#   cat("\n\nii =", ii, "\n")
    alldat = transform(alldat, usey=y-offsety)
    iloc = ifelse(ii==1, with(alldat, median(y)), 1.0) # Well-chosen!
    mydf = ifelse(ii==1, 5, 3)  # Maybe less smoothing will help
    lloc = ifelse(ii==1, "identity", "loge")  # 2nd value must be "loge"
    fit3 = vglm(usey ~ ns(x, df=mydf), data=alldat, trace=TRUE,
                fam=alaplace1(tau=usetau[ii], lloc=lloc, iloc=iloc))
    answer[,ii] = (if(ii==1) 0 else answer[,ii-1]) + fitted(fit3)
    alldat = transform(alldat, offsety=answer[,ii])
}

# Plot the results.
## Not run: 
##D with(alldat, plot(x, y, col="blue",
##D      main=paste("Noncrossing and nonparallel; tau =",
##D                 paste(mytau, collapse=", "))))
##D with(alldat, matlines(x, answer, col="red", lty=1))
##D 
##D # Zoom in near the origin.
##D with(alldat, plot(x, y, col="blue", xlim=c(0, 0.2), ylim=0:1,
##D      main=paste("Noncrossing and nonparallel; tau =",
##D                 paste(mytau, collapse=", "))))
##D with(alldat, matlines(x, answer, col="red", lty=1))
## End(Not run)



